@@ -19,7 +19,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
 #include <string.h>
@@ -29,8 +29,8 @@
 #include "gnome-da-capplet.h"
 #include "gnome-da-xml.h"
 #include "gnome-da-item.h"
+#include "capplet-util.h"
 
-/* TODO: add support for themeable icons also for combo boxes */
 /* TODO: it doesn't use GConfPropertyEditor, use it when/if moved to control-center */
 
 enum
@@ -44,23 +44,7 @@ static void
 close_cb (GtkWidget *window, gint response, gpointer user_data)
 {
     if (response == GTK_RESPONSE_HELP) {
-	GError *error = NULL;
-
-	/* capplet_help (GTK_WINDOW (dialog), "user-guide.xml", "goscustdoc-2"); */
-	gnome_help_display_desktop (NULL, "user-guide", "user-guide.xml",
-				    "goscustdoc-2", &error);
-	if (error != NULL) {
-	    GtkWidget *dialog;
-
-	    dialog = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-					     _("Could not display help"));
-	    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						      _("Please make sure that the applet "
-							"is properly installed"));
-	    gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
-	    gtk_dialog_run (GTK_DIALOG (dialog));
-	    gtk_widget_destroy (dialog);
-	}
+	capplet_help (GTK_WINDOW (window), "user-guide.xml", "prefs-preferredapps");
     }
     else {
 	gtk_widget_destroy (window);
@@ -102,21 +86,12 @@ static void
 terminal_checkbutton_toggled_cb (GtkToggleButton *togglebutton, GnomeDACapplet *capplet)
 {
     gboolean is_active;
-    GConfChangeSet *cs;
     GError *error = NULL;
 
     is_active = gtk_toggle_button_get_active (togglebutton);
 
     if (GTK_WIDGET (togglebutton) == capplet->web_browser_terminal_checkbutton) {
-	cs = gconf_change_set_new ();
-
-	gconf_change_set_set_bool (cs, DEFAULT_APPS_KEY_HTTP_NEEDS_TERM, is_active);
-	gconf_change_set_set_bool (cs, DEFAULT_APPS_KEY_HTTPS_NEEDS_TERM, is_active);
-	gconf_change_set_set_bool (cs, DEFAULT_APPS_KEY_UNKNOWN_NEEDS_TERM, is_active);
-	gconf_change_set_set_bool (cs, DEFAULT_APPS_KEY_ABOUT_NEEDS_TERM, is_active);
-
-	gconf_client_commit_change_set (capplet->gconf, cs, TRUE, &error);
-	gconf_change_set_unref (cs);
+	gconf_client_set_bool (capplet->gconf, DEFAULT_APPS_KEY_HTTP_NEEDS_TERM, is_active, &error);
     }
     else if (GTK_WIDGET (togglebutton) == capplet->mail_reader_terminal_checkbutton) {
 	gconf_client_set_bool (capplet->gconf, DEFAULT_APPS_KEY_MAILER_NEEDS_TERM, is_active, &error);
@@ -140,20 +115,6 @@ set_icon (GtkImage *image, GtkIconTheme *theme, const char *name)
     }
 }
 
-static struct {
-    const gchar *name;
-    const gchar *icon;
-} icons[] = {
-    { "web_browser_image", "web-browser"      },
-    { "mail_reader_image", "stock_mail-open"  },
-/*    { "messenger_image",   "im"               },
- *    { "image_image",       "image-viewer"     },
- *    { "sound_image",       "gnome-audio"      },
- *    { "video_image",       "gnome-multimedia" },
- *    { "text_image",        "text-editor"      }, */
-    { "terminal_image",    "gnome-terminal"   }
-};
-
 static void
 web_radiobutton_toggled_cb (GtkToggleButton *togglebutton, GnomeDACapplet *capplet)
 {
@@ -248,7 +209,7 @@ web_combo_changed_cb (GtkComboBox *combo, GnomeDACapplet *capplet)
     gtk_widget_set_sensitive (capplet->new_win_radiobutton, has_net_remote);
     gtk_widget_set_sensitive (capplet->new_tab_radiobutton, has_net_remote);
 
-    gtk_entry_set_editable (GTK_ENTRY (capplet->web_browser_command_entry), is_custom_active);
+    gtk_editable_set_editable (GTK_EDITABLE (capplet->web_browser_command_entry), is_custom_active);
     gtk_widget_set_sensitive (capplet->web_browser_command_label, is_custom_active);
     gtk_widget_set_sensitive (capplet->web_browser_terminal_checkbutton, is_custom_active);
 }
@@ -297,7 +258,7 @@ mail_combo_changed_cb (GtkComboBox *combo, GnomeDACapplet *capplet)
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (capplet->mail_reader_terminal_checkbutton),
 				  gconf_client_get_bool (capplet->gconf, DEFAULT_APPS_KEY_MAILER_NEEDS_TERM, NULL));
 
-    gtk_entry_set_editable (GTK_ENTRY (capplet->mail_reader_command_entry), is_custom_active);
+    gtk_editable_set_editable (GTK_EDITABLE (capplet->mail_reader_command_entry), is_custom_active);
     gtk_widget_set_sensitive (capplet->mail_reader_command_label, is_custom_active);
     gtk_widget_set_sensitive (capplet->mail_reader_terminal_checkbutton, is_custom_active);
 }
@@ -346,22 +307,83 @@ terminal_combo_changed_cb (GtkComboBox *combo, GnomeDACapplet *capplet)
     gtk_entry_set_text (GTK_ENTRY (capplet->terminal_exec_flag_entry),
 			gconf_client_get_string (capplet->gconf, DEFAULT_APPS_KEY_TERMINAL_EXEC_ARG, NULL));
 
-    gtk_entry_set_editable (GTK_ENTRY (capplet->terminal_command_entry), is_custom_active);
+    gtk_editable_set_editable (GTK_EDITABLE (capplet->terminal_command_entry), is_custom_active);
     gtk_widget_set_sensitive (capplet->terminal_command_label, is_custom_active);
-    gtk_entry_set_editable (GTK_ENTRY (capplet->terminal_exec_flag_entry), is_custom_active);
+    gtk_editable_set_editable (GTK_EDITABLE (capplet->terminal_exec_flag_entry), is_custom_active);
     gtk_widget_set_sensitive (capplet->terminal_exec_flag_label, is_custom_active);
 }
 
 static void
-theme_changed_cb (GtkIconTheme *theme, GladeXML *xml)
+refresh_combo_box_icons (GtkIconTheme *theme, GtkComboBox *combo_box, GList *app_list)
+{
+    GList *entry;
+    GnomeDAItem *item;
+    GtkTreeModel *model;
+    GtkTreeIter iter;
+    GdkPixbuf *pixbuf;
+
+    for (entry = app_list; entry != NULL; entry = g_list_next (entry)) {
+	item = (GnomeDAItem *) entry->data;
+
+	model = gtk_combo_box_get_model (combo_box);
+
+	if (item->icon_path && gtk_tree_model_get_iter_from_string (model, &iter, item->icon_path)) {
+	    pixbuf = gtk_icon_theme_load_icon (theme, item->icon_name, 22, 0, NULL);
+
+	    gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+				PIXBUF_COL, pixbuf,
+				-1);
+
+	    if (pixbuf)
+		g_object_unref (pixbuf);
+	}
+    }
+}
+
+static struct {
+    const gchar *name;
+    const gchar *icon;
+} icons[] = {
+    { "web_browser_image", "web-browser"      },
+    { "mail_reader_image", "stock_mail-open"  },
+/*    { "messenger_image",   "im"               },
+ *    { "image_image",       "image-viewer"     },
+ *    { "sound_image",       "gnome-audio"      },
+ *    { "video_image",       "gnome-multimedia" },
+ *    { "text_image",        "text-editor"      }, */
+    { "terminal_image",    "gnome-terminal"   }
+};
+
+static void
+theme_changed_cb (GtkIconTheme *theme, GnomeDACapplet *capplet)
 {
     GtkWidget *icon;
     gint i;
 
     for (i = 0; i < G_N_ELEMENTS (icons); i++) {
-	icon = glade_xml_get_widget (xml, icons[i].name);
+	icon = glade_xml_get_widget (capplet->xml, icons[i].name);
 	set_icon (GTK_IMAGE (icon), theme, icons[i].icon);
     }
+
+    refresh_combo_box_icons (theme, GTK_COMBO_BOX (capplet->web_combo_box), capplet->web_browsers);
+    refresh_combo_box_icons (theme, GTK_COMBO_BOX (capplet->mail_combo_box), capplet->mail_readers);
+    refresh_combo_box_icons (theme, GTK_COMBO_BOX (capplet->term_combo_box), capplet->terminals);
+}
+
+static void
+screen_changed_cb (GtkWidget *widget, GdkScreen *screen, GnomeDACapplet *capplet)
+{
+    GtkIconTheme *theme;
+
+    theme = gtk_icon_theme_get_for_screen (screen);
+
+    if (capplet->icon_theme != NULL) {
+	g_signal_handlers_disconnect_by_func (capplet->icon_theme, theme_changed_cb, capplet);
+    }
+    g_signal_connect (theme, "changed", G_CALLBACK (theme_changed_cb), capplet);
+    theme_changed_cb (theme, capplet);
+
+    capplet->icon_theme = theme;
 }
 
 static gint
@@ -413,7 +435,7 @@ web_browser_update_combo_box (GnomeDACapplet *capplet, const gchar *command)
     /* TODO: Remove when GConfPropertyEditor will be used */
     gtk_entry_set_text (GTK_ENTRY (capplet->web_browser_command_entry), command);
 
-    gtk_entry_set_editable (GTK_ENTRY (capplet->web_browser_command_entry), is_custom_active);
+    gtk_editable_set_editable (GTK_EDITABLE (capplet->web_browser_command_entry), is_custom_active);
     gtk_widget_set_sensitive (capplet->web_browser_command_label, is_custom_active);
     gtk_widget_set_sensitive (capplet->web_browser_terminal_checkbutton, is_custom_active);
 
@@ -488,7 +510,7 @@ mail_reader_update_combo_box (GnomeDACapplet *capplet, const gchar *command)
 
     gtk_entry_set_text (GTK_ENTRY (capplet->mail_reader_command_entry), command);
 
-    gtk_entry_set_editable (GTK_ENTRY (capplet->mail_reader_command_entry), is_custom_active);
+    gtk_editable_set_editable (GTK_EDITABLE (capplet->mail_reader_command_entry), is_custom_active);
     gtk_widget_set_sensitive (capplet->mail_reader_command_label, is_custom_active);
     gtk_widget_set_sensitive (capplet->mail_reader_terminal_checkbutton, is_custom_active);
 
@@ -517,9 +539,9 @@ terminal_update_combo_box (GnomeDACapplet *capplet, const gchar *command)
 
     gtk_entry_set_text (GTK_ENTRY (capplet->terminal_command_entry), command);
 
-    gtk_entry_set_editable (GTK_ENTRY (capplet->terminal_command_entry), is_custom_active);
+    gtk_editable_set_editable (GTK_EDITABLE (capplet->terminal_command_entry), is_custom_active);
     gtk_widget_set_sensitive (capplet->terminal_command_label, is_custom_active);
-    gtk_entry_set_editable (GTK_ENTRY (capplet->terminal_exec_flag_entry), is_custom_active);
+    gtk_editable_set_editable (GTK_EDITABLE (capplet->terminal_exec_flag_entry), is_custom_active);
     gtk_widget_set_sensitive (capplet->terminal_exec_flag_label, is_custom_active);
 
     if (gtk_combo_box_get_active (GTK_COMBO_BOX (capplet->term_combo_box)) != index)
@@ -532,6 +554,7 @@ web_gconf_changed_cb (GConfClient *client, guint id, GConfEntry *entry, GnomeDAC
     GConfValue *value;
     GConfChangeSet *cs;
     GError *error = NULL;
+    GList *list_entry;
 
     g_return_if_fail (gconf_entry_get_key (entry) != NULL);
 
@@ -539,6 +562,7 @@ web_gconf_changed_cb (GConfClient *client, guint id, GConfEntry *entry, GnomeDAC
 	return;
 
     if (strcmp (entry->key, DEFAULT_APPS_KEY_HTTP_EXEC) == 0) {
+	gchar* short_browser;
 	web_browser_update_combo_box (capplet, gconf_value_get_string (value));
 	web_browser_update_radio_buttons (capplet, gconf_value_get_string (value));
 
@@ -547,6 +571,21 @@ web_gconf_changed_cb (GConfClient *client, guint id, GConfEntry *entry, GnomeDAC
 	gconf_change_set_set (cs, DEFAULT_APPS_KEY_HTTPS_EXEC, value);
 	gconf_change_set_set (cs, DEFAULT_APPS_KEY_UNKNOWN_EXEC, value);
 	gconf_change_set_set (cs, DEFAULT_APPS_KEY_ABOUT_EXEC, value);
+	short_browser = g_strndup(gconf_value_get_string(value),
+				  strstr(gconf_value_get_string(value), " ") -
+				  gconf_value_get_string(value));
+	gconf_change_set_set_string (cs, DEFAULT_APPS_KEY_BROWSER_EXEC, short_browser);
+	g_free(short_browser);
+
+	list_entry = g_list_find_custom (capplet->web_browsers,
+					 gconf_value_get_string (value),
+					 (GCompareFunc) web_item_comp);
+
+	if (list_entry) {
+	    GnomeDAWebItem *item = (GnomeDAWebItem *) list_entry->data;
+
+	    gconf_change_set_set_bool (cs, DEFAULT_APPS_KEY_BROWSER_NREMOTE, item->netscape_remote);
+	}
 
 	gconf_client_commit_change_set (capplet->gconf, cs, TRUE, &error);
 
@@ -568,6 +607,7 @@ web_gconf_changed_cb (GConfClient *client, guint id, GConfEntry *entry, GnomeDAC
 	gconf_change_set_set (cs, DEFAULT_APPS_KEY_HTTPS_NEEDS_TERM, value);
 	gconf_change_set_set (cs, DEFAULT_APPS_KEY_UNKNOWN_NEEDS_TERM, value);
 	gconf_change_set_set (cs, DEFAULT_APPS_KEY_ABOUT_NEEDS_TERM, value);
+	gconf_change_set_set (cs, DEFAULT_APPS_KEY_BROWSER_NEEDS_TERM, value);
 
 	gconf_client_commit_change_set (capplet->gconf, cs, TRUE, &error);
 
@@ -644,6 +684,10 @@ fill_combo_box (GtkIconTheme *theme, GtkComboBox *combo_box, GList *app_list)
     GdkPixbuf *pixbuf;
     gchar *label;
 
+    if (theme == NULL) {
+	theme = gtk_icon_theme_get_default ();
+    }
+
     gtk_combo_box_set_row_separator_func (combo_box, is_separator,
 					  GINT_TO_POINTER (g_list_length (app_list)), NULL);
 
@@ -678,6 +722,8 @@ fill_combo_box (GtkIconTheme *theme, GtkComboBox *combo_box, GList *app_list)
 			    TEXT_COL, label,
 			    -1);
 
+	item->icon_path = gtk_tree_model_get_string_from_iter (model, &iter);
+
 	if (pixbuf)
 	    g_object_unref (pixbuf);
 	g_free (label);
@@ -696,13 +742,12 @@ static void
 show_dialog (GnomeDACapplet *capplet)
 {
     GConfValue *value;
-    GtkIconTheme *theme;
 
     if (g_file_test (GLADEDIR "/gnome-default-applications-properties.glade", G_FILE_TEST_EXISTS) != FALSE) {
-	capplet->xml = glade_xml_new (GLADEDIR "/gnome-default-applications-properties.glade", NULL, PACKAGE);
+	capplet->xml = glade_xml_new (GLADEDIR "/gnome-default-applications-properties.glade", NULL, NULL);
     }
     else {
-	capplet->xml = glade_xml_new ("./gnome-default-applications-properties.glade", NULL, PACKAGE);
+	capplet->xml = glade_xml_new ("./gnome-default-applications-properties.glade", NULL, NULL);
     }
 
     if (capplet->xml == NULL) {
@@ -722,10 +767,6 @@ show_dialog (GnomeDACapplet *capplet)
     capplet->window = glade_xml_get_widget (capplet->xml, "preferred_apps_dialog");
     g_signal_connect (capplet->window, "response", G_CALLBACK (close_cb), NULL);
 
-    theme = gtk_icon_theme_get_default ();
-    g_signal_connect (theme, "changed", G_CALLBACK (theme_changed_cb), capplet->xml);
-    theme_changed_cb (theme, capplet->xml);
-
     capplet->web_browser_command_entry = glade_xml_get_widget (capplet->xml, "web_browser_command_entry");
     capplet->web_browser_command_label = glade_xml_get_widget (capplet->xml, "web_browser_command_label");
     capplet->web_browser_terminal_checkbutton = glade_xml_get_widget (capplet->xml, "web_browser_terminal_checkbutton");
@@ -746,9 +787,12 @@ show_dialog (GnomeDACapplet *capplet)
     capplet->mail_combo_box = glade_xml_get_widget (capplet->xml, "mail_reader_combobox");
     capplet->term_combo_box = glade_xml_get_widget (capplet->xml, "terminal_combobox");
 
-    fill_combo_box (theme, GTK_COMBO_BOX (capplet->web_combo_box), capplet->web_browsers);
-    fill_combo_box (theme, GTK_COMBO_BOX (capplet->mail_combo_box), capplet->mail_readers);
-    fill_combo_box (theme, GTK_COMBO_BOX (capplet->term_combo_box), capplet->terminals);
+    g_signal_connect (capplet->window, "screen-changed", G_CALLBACK (screen_changed_cb), capplet);
+    screen_changed_cb (capplet->window, gdk_screen_get_default (), capplet);
+
+    fill_combo_box (capplet->icon_theme, GTK_COMBO_BOX (capplet->web_combo_box), capplet->web_browsers);
+    fill_combo_box (capplet->icon_theme, GTK_COMBO_BOX (capplet->mail_combo_box), capplet->mail_readers);
+    fill_combo_box (capplet->icon_theme, GTK_COMBO_BOX (capplet->term_combo_box), capplet->terminals);
 
     /* update ui to gconf content */
     value = gconf_client_get (capplet->gconf, DEFAULT_APPS_KEY_HTTP_EXEC, NULL);
@@ -807,15 +851,17 @@ main (int argc, char **argv)
 
     capplet = g_new0 (GnomeDACapplet, 1);
 
+#ifdef ENABLE_NLS
+    bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+    textdomain (GETTEXT_PACKAGE);
+#endif
+
     gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
 			GNOME_PARAM_NONE);
 
     glade_init ();
 
-    bindtextdomain (PACKAGE, GNOMELOCALEDIR);
-    bind_textdomain_codeset (PACKAGE, "UTF-8");
-    textdomain (PACKAGE);
-
     capplet->gconf = gconf_client_get_default ();
 
     gconf_client_add_dir (capplet->gconf, "/desktop/gnome/applications/browser", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
